@@ -22,6 +22,7 @@
 #include "gtkprogresstrackerprivate.h"
 #include "hdy-animation-private.h"
 #include "hdy-cairo-private.h"
+#include "hdy-css-private.h"
 
 /**
  * SECTION:hdy-squeezer
@@ -691,9 +692,13 @@ hdy_squeezer_draw_crossfade (GtkWidget *widget,
   if (self->last_visible_surface != NULL) {
     gint width_diff = gtk_widget_get_allocated_width (widget) - self->last_visible_surface_allocation.width;
     gint height_diff = gtk_widget_get_allocated_height (widget) - self->last_visible_surface_allocation.height;
+    float xalign = self->xalign;
+
+    if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+      xalign = 1 - xalign;
 
     cairo_set_source_surface (cr, self->last_visible_surface,
-                              width_diff * self->xalign,
+                              width_diff * xalign,
                               height_diff * self->yalign);
     cairo_set_operator (cr, CAIRO_OPERATOR_ADD);
     cairo_paint_with_alpha (cr, MAX (1.0 - progress, 0));
@@ -779,6 +784,8 @@ hdy_squeezer_size_allocate (GtkWidget     *widget,
   GList *l;
   GtkAllocation child_allocation;
 
+  hdy_css_size_allocate (widget, allocation);
+
   gtk_widget_set_allocation (widget, allocation);
 
   for (l = self->children; l != NULL; l = l->next) {
@@ -828,12 +835,22 @@ hdy_squeezer_size_allocate (GtkWidget     *widget,
 
   if (self->last_visible_child != NULL) {
     int min, nat;
-    gtk_widget_get_preferred_width (self->last_visible_child->widget, &min, &nat);
-    child_allocation.width = MAX (min, allocation->width);
-    gtk_widget_get_preferred_height_for_width (self->last_visible_child->widget,
-                                               child_allocation.width,
-                                               &min, &nat);
-    child_allocation.height = MAX (min, allocation->height);
+
+    if (self->orientation == GTK_ORIENTATION_HORIZONTAL) {
+      gtk_widget_get_preferred_width (self->last_visible_child->widget, &min, &nat);
+      child_allocation.width = MAX (min, allocation->width);
+      gtk_widget_get_preferred_height_for_width (self->last_visible_child->widget,
+                                                 child_allocation.width,
+                                                 &min, &nat);
+      child_allocation.height = MAX (min, allocation->height);
+    } else {
+      gtk_widget_get_preferred_height (self->last_visible_child->widget, &min, &nat);
+      child_allocation.height = MAX (min, allocation->height);
+      gtk_widget_get_preferred_width_for_height (self->last_visible_child->widget,
+                                                 child_allocation.height,
+                                                 &min, &nat);
+      child_allocation.width = MAX (min, allocation->width);
+    }
 
     gtk_widget_size_allocate (self->last_visible_child->widget, &child_allocation);
   }
@@ -843,22 +860,25 @@ hdy_squeezer_size_allocate (GtkWidget     *widget,
 
   if (self->visible_child) {
     int min, nat;
-    GtkAlign valign;
 
     gtk_widget_get_preferred_height_for_width (self->visible_child->widget,
-                                               allocation->width,
-                                               &min, &nat);
-    if (self->interpolate_size) {
-      valign = gtk_widget_get_valign (self->visible_child->widget);
-      child_allocation.height = MAX (nat, allocation->height);
-      if (valign == GTK_ALIGN_END &&
-          child_allocation.height > allocation->height)
-        child_allocation.y -= nat - allocation->height;
-      else if (valign == GTK_ALIGN_CENTER &&
-               child_allocation.height > allocation->height)
-        child_allocation.y -= (nat - allocation->height) / 2;
+                                               allocation->width, &min, &nat);
+    child_allocation.width = MAX (child_allocation.width, min);
+
+    gtk_widget_get_preferred_width_for_height (self->visible_child->widget,
+                                               allocation->height, &min, &nat);
+    child_allocation.height = MAX (child_allocation.height, min);
+
+    if (child_allocation.width > allocation->width) {
+      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+        child_allocation.x = (allocation->width - child_allocation.width) * (1 - self->xalign);
+      else
+        child_allocation.x = (allocation->width - child_allocation.width) * self->xalign;
     }
 
+    if (child_allocation.height > allocation->height)
+      child_allocation.y = (allocation->height - child_allocation.height) * self->yalign;
+
     gtk_widget_size_allocate (self->visible_child->widget, &child_allocation);
   }
 }
@@ -932,6 +952,8 @@ hdy_squeezer_measure (GtkWidget      *widget,
       *natural = hdy_lerp (self->last_visible_widget_width, *natural, t);
     }
   }
+
+  hdy_css_measure (widget, orientation, minimum, natural);
 }
 
 static void
@@ -1127,7 +1149,7 @@ hdy_squeezer_class_init (HdySqueezerClass *klass)
   /**
    * HdySqueezer:xalign:
    *
-   * The xalign property determines the horizontal aligment of the children
+   * The xalign property determines the horizontal alignment of the children
    * inside the squeezer's size allocation.
    * Compare this to #GtkWidget:halign, which determines how the squeezer's size
    * allocation is positioned in the space available for the squeezer.
@@ -1149,7 +1171,7 @@ hdy_squeezer_class_init (HdySqueezerClass *klass)
   /**
    * HdySqueezer:yalign:
    *
-   * The yalign property determines the vertical aligment of the children inside
+   * The yalign property determines the vertical alignment of the children inside
    * the squeezer's size allocation.
    * Compare this to #GtkWidget:valign, which determines how the squeezer's size
    * allocation is positioned in the space available for the squeezer.
